/*jshint esversion: 6 */
	import * as THREE from 'three';
	import {FontLoader} from 'three/addons/loaders/FontLoader.js';
	import {TextGeometry} from 'three/addons/geometries/TextGeometry.js';

	// gun rack
	export function gun(param) {
		const width=800, height=2000, depth=800;
		const gunObject = new THREE.Object3D();
		const gunmaterial = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
		const gungeometry = new THREE.BoxGeometry(width, height, depth);
		const gunmesh = new THREE.Mesh(gungeometry, gunmaterial);
		gunObject.add(gunmesh);
		return gunObject;
	}

	// quadrupole
	export function quadrupoleboosterfast(param) {
		const length=300;
		const quadrupoleMaster = new THREE.Object3D();
		const dmaterial = new THREE.MeshLambertMaterial({color: 0xe05040});
		const d1geometry = new THREE.BoxGeometry(550, 500, length);
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		quadrupoleMaster.add(d1mesh);				
		return quadrupoleMaster;
	}

	// BPM (beam position monitor)
	export function bpm(param) {
		const width=300, height=300, depth=50, transverse=100;
		const bpmObject = new THREE.Object3D();
		const bpmmaterial = new THREE.MeshLambertMaterial({color: 0x8000ff});
		const bpmgeometryh = new THREE.BoxGeometry(width, transverse, depth);
		const bpmmeshh = new THREE.Mesh(bpmgeometryh, bpmmaterial);
		bpmObject.add(bpmmeshh);
		const bpmgeometryv = new THREE.BoxGeometry(transverse, height, depth);
		const bpmmeshv = new THREE.Mesh(bpmgeometryv, bpmmaterial);
		bpmObject.add(bpmmeshv);
		return bpmObject;
	}

	// flsc
	export function flscfast(param) {
		const length=700;
		const flscSR = new THREE.Object3D();
		const flscContiner = new THREE.Object3D();
		const materialGrey = new THREE.MeshLambertMaterial({color: 0xf0f0f0});

		const d1geometry = new THREE.CylinderGeometry(25, 25, 500, 15);
		const d1mesh = new THREE.Mesh(d1geometry, materialGrey);
		flscContiner.add(d1mesh);				
		flscContiner.rotateX(Math.PI * 1);

		// zigrinatura
		const d4geometry = new THREE.CylinderGeometry(70, 70, 160, 20);
		const textureLoader = new THREE.TextureLoader();
		const texture = textureLoader.load('./components/flsc_texture.png');	
		texture.wrapS = texture.wrapT = THREE.RepeatWrapping;
		texture.repeat.set(5, 40);		
		const material = new THREE.MeshBasicMaterial({ map: texture });		
		const d4mesh = new THREE.Mesh(d4geometry, material);
		d4mesh.rotateX(Math.PI * 1);
		d4mesh.rotateY(Math.PI * 1);
		d4mesh.rotateZ(Math.PI * 1);
		d4mesh.position.set(0, 80 , 0);			
		flscContiner.add(d4mesh);		

		// cil3
		const dmateria6 = new THREE.MeshLambertMaterial({color: 0x000000});
		const d6geometry = new THREE.CylinderGeometry(50, 50, 60, 15);
		const d6mesh = new THREE.Mesh(d6geometry, dmateria6);
		d6mesh.rotateX(Math.PI * 1);
		d6mesh.rotateY(Math.PI * 1);
		d6mesh.rotateZ(Math.PI * 1);
		d6mesh.position.set(0, -30 , 0);			
		flscContiner.add(d6mesh);		
		
		// box gray
		const d7geometry = new THREE.BoxGeometry(100, 100, 100);
		const dmaterial7= new THREE.MeshLambertMaterial({color: 0x405040});
		const d7mesh = new THREE.Mesh(d7geometry, dmaterial7);
		d7mesh.position.set(0, -110, 0);		
		flscContiner.add(d7mesh);				
				
		// box orange
		const d9geometry = new THREE.BoxGeometry(100, 120, 100);
		const dmaterial9= new THREE.MeshLambertMaterial({color: 0xffA500});
		const d9mesh = new THREE.Mesh(d9geometry, dmaterial9);
		d9mesh.position.set(0, -220, 0);		
		flscContiner.add(d9mesh);	

		flscContiner.position.set(0, 280, 0);
		flscSR.add(flscContiner);
		flscSR.rotateY(Math.PI * -0.5);
		return flscSR;
	}

	// corrector
	export function correctorboosterfast(param) {
		const width=250, height=250, depth=200;
		const correctorObject = new THREE.Object3D();
		const correctormaterial = new THREE.MeshLambertMaterial({color: 0x4040b0});
		const correctorgeometry = new THREE.BoxGeometry(width, height, depth);
		const correctormesh = new THREE.Mesh(correctorgeometry, correctormaterial);
		correctorObject.add(correctormesh);
		return correctorObject;
	}


	// cavity for linac
	export function cavitylinac(param) {
		const length=3000;
		const cavitylinacMaster = new THREE.Object3D();
		const dmaterial = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
		const d1geometry = new THREE.CylinderGeometry(400, 400, length, 25);
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		cavitylinacMaster.add(d1mesh);				
		cavitylinacMaster.rotateX(Math.PI * 0.5);
		cavitylinacMaster.rotatedX = true;
		return cavitylinacMaster;
	}


	// Bending Dipole
	export function dipoleboosterfast(length = 1915) {
		const dipoleMaster = new THREE.Object3D();
		const dmaterial = new THREE.MeshLambertMaterial({color: 0x009000});
		const d1geometry = new THREE.BoxGeometry(550, 400, length);
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		d1mesh.position.set(100, 100, 0);
		dipoleMaster.add(d1mesh);				
		return dipoleMaster;
	}


	// Sextupole
	export function sextupoleboosterfast(param) {
		const length=200;
		const sextupoleMaster = new THREE.Object3D();
		const dmaterial = new THREE.MeshLambertMaterial({color: 0xffff00});
		const d1geometry = new THREE.CylinderGeometry(300, 300, length, 6);
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		d1mesh.rotateX(Math.PI * 0.5);
		d1mesh.rotateY(Math.PI * 0.5);
		sextupoleMaster.add(d1mesh);				
		return sextupoleMaster;
	}
	// septum
		export function spte(param) {
		const length = param && typeof param.length == "number"? param.length: 1000;
			const septumSR = new THREE.Object3D();
			const dmaterial = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
			const d1geometry = new THREE.CylinderGeometry(200, 200, length, 25);
			const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
			septumSR.add(d1mesh);				
			septumSR.rotateX(Math.PI * 0.5);
			septumSR.rotatedX = true;	

			// left
			const dmateria2 = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
			const d2geometry = new THREE.CylinderGeometry(250, 250, 100, 25);
			const d2mesh = new THREE.Mesh(d2geometry, dmateria2);
			d2mesh.position.set(0, -length/2-50, 0);
			septumSR.add(d2mesh);

			// right
			const dmateria3 = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
			const d3geometry = new THREE.CylinderGeometry(250, 250, 100, 25);
			const d3mesh = new THREE.Mesh(d3geometry, dmateria3);
			d3mesh.position.set(0, length/2+50, 0);
			septumSR.add(d3mesh);

		
			septumSR.scale.setY(0.7);

			return septumSR;
		}

	// kicker
	export function kicker(param) {
		const length=300;
		const septumMaster = new THREE.Object3D();

		const d1geometry = new THREE.BoxGeometry(710, 500, 500);
		const dmaterial = new THREE.MeshLambertMaterial({color: 0x405040});
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		d1mesh.position.set(0, -20, 330);
		septumMaster.add(d1mesh);				

		//add quad2
		const d2geometry = new THREE.BoxGeometry(710, 270, 300);
		const dmaterial2= new THREE.MeshLambertMaterial({color: 0x405040});
		const d2mesh = new THREE.Mesh(d2geometry, dmaterial2);
		d2mesh.position.set(0, -135, 80);
		septumMaster.add(d2mesh);

		//add sest
		const d3geometry = new THREE.CylinderGeometry(300, 300, 700, 6);
		const dmateria3 = new THREE.MeshLambertMaterial({color: 0xffff00});
		const textureLoader = new THREE.TextureLoader();
		const texture = textureLoader.load('./components/kicker_texture.png');
		texture.wrapS = texture.wrapT = THREE.RepeatWrapping;
		texture.repeat.set(5, 40);
		const material = new THREE.MeshBasicMaterial({ map: texture });
		const d3mesh = new THREE.Mesh(d3geometry, material);
		d3mesh.rotateX(Math.PI * 1);
		d3mesh.rotateY(Math.PI * 1);
		d3mesh.rotateZ(Math.PI * 1.5);
		d3mesh.position.set(0, 0 , 230);
		septumMaster.add(d3mesh);				

		//capsule
		const geometry = new THREE.CapsuleGeometry( 50, 50, 40, 80 ); 
		const material_cap = new THREE.MeshBasicMaterial( {color: 0xff0000} ); 
		const d4mesh = new THREE.Mesh( geometry, material_cap ); 
		d4mesh.position.set(0, 240 , 480);
		septumMaster.add(d4mesh);

		septumMaster.rotateY(Math.PI * 0.5);
		return septumMaster;
	}

	// lens
	export function lens(param) {
		const lensContiner = new THREE.Object3D();
		const lensObject = new THREE.Object3D();
		
		//cylinder mid
		const cylgeometry = new THREE.CylinderGeometry(105, 105, 30, 20);
		const cylmaterial = new THREE.MeshLambertMaterial({color: 0xc7c7c7});
		const cylmesh = new THREE.Mesh(cylgeometry, cylmaterial);
		cylmesh.rotateX(Math.PI * 1);
		cylmesh.rotateY(Math.PI * 1.0);
		cylmesh.rotateZ(Math.PI * 1.5);
		cylmesh.position.set(0, 0 , 0);			
		lensObject.add(cylmesh);			
		
		//cylinder center
		const cyl2geometry = new THREE.CylinderGeometry(75, 75, 31, 20);
		const cyl2material = new THREE.MeshLambertMaterial({color: 0x737373});
		const cyl2mesh = new THREE.Mesh(cyl2geometry, cyl2material);
		cyl2mesh.rotateX(Math.PI * 1);
		cyl2mesh.rotateY(Math.PI * 1.0);
		cyl2mesh.rotateZ(Math.PI * 1.5);
		cyl2mesh.position.set(0, 0 , 0);			
		lensObject.add(cyl2mesh);			
		
		// base1
		const basegeometry = new THREE.BoxGeometry(29, 160, 60 );
		const basemesh = new THREE.Mesh(basegeometry, cylmaterial);
		basemesh.position.set(0, -70, 0);
		lensObject.add(basemesh);		
		
			
		lensObject.scale.setX(1);
		lensObject.scale.setY(1);
		lensObject.scale.setZ(1);
		lensObject.position.set(0, 0, 0);
		lensContiner.add(lensObject);
		lensContiner.rotateY(Math.PI * 0.5);		
		
		return lensContiner;
	}
	// cavity
	export function cavity(param) {
		const sphereRadius=300, sphereFactor=0.5, cylinderRadius=40, cylinderHeight=600;
		const cavityObject = new THREE.Object3D();
		const cavitymaterial = new THREE.MeshLambertMaterial({color: 0xb0b0b0});
		const cavitygeometry = new THREE.SphereGeometry(sphereRadius, 32, 32);
		const cavitymesh = new THREE.Mesh(cavitygeometry, cavitymaterial);
		cavitymesh.scale.setZ(sphereFactor);
		cavityObject.add(cavitymesh);
		const rfGeometry = new THREE.CylinderGeometry(cylinderRadius, cylinderRadius,  cylinderHeight, 20, 2, false, 0, Math.PI * 2);
		const rfMesh = new THREE.Mesh(rfGeometry, cavitymaterial);
		// rfMesh.rotateX(Math.PI * 0.5);
		rfMesh.position.set(0, sphereRadius, 0);
		cavityObject.add(rfMesh);
		return cavityObject;
	}

	// 3hc
	export function cavity_3hc(param) {
		const threeHCSR = new THREE.Object3D();
		const dmaterial = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
		const dmaterial2 = new THREE.MeshLambertMaterial({color: 0xffffff});
		const d1geometry = new THREE.CylinderGeometry(700, 700, 1500, 35);
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		threeHCSR.add(d1mesh);				
		threeHCSR.rotateX(Math.PI * 0.5);
		threeHCSR.rotatedX = true;	

		// left
		const lrgeometry = new THREE.CylinderGeometry(750, 750, 100, 65);
		const d2mesh = new THREE.Mesh(lrgeometry, dmaterial);
		d2mesh.position.set(0, -1500/2-50, 0);
		threeHCSR.add(d2mesh);

		// right
		const d3mesh = new THREE.Mesh(lrgeometry, dmaterial);
		d3mesh.position.set(0, 1500/2+50, 0);
		threeHCSR.add(d3mesh);

		// up1
		const udgeometry = new THREE.CylinderGeometry(250, 250, 700, 25);
		const d4mesh = new THREE.Mesh(udgeometry, dmaterial);
		d4mesh.rotateX(Math.PI * 0.5);
		d4mesh.rotateY(Math.PI * 1);
		d4mesh.rotateZ(Math.PI * 1);
		d4mesh.position.set(0, -400 , -730);		
		threeHCSR.add(d4mesh);

		// up2
		const udgeometry2 = new THREE.CylinderGeometry(150, 150, 700, 25);
		const d5mesh = new THREE.Mesh(udgeometry2, dmaterial);
		d5mesh.rotateX(Math.PI * 0.5);
		d5mesh.rotateY(Math.PI * 1);
		d5mesh.rotateZ(Math.PI * 1);
		d5mesh.position.set(0, 400 , -730);		
		threeHCSR.add(d5mesh);

		
		//tubo1
		const tubegeometry1 = new THREE.CylinderGeometry(30, 30, 1100, 25);
		const d12mesh = new THREE.Mesh(tubegeometry1, dmaterial);
		d12mesh.rotateX(Math.PI * 0.5);
		d12mesh.rotateY(Math.PI * 0.5);
		d12mesh.rotateZ(Math.PI * 0.5);
		d12mesh.position.set(-430, -150 , -930);		
		threeHCSR.add(d12mesh);	
		
		//tubo2
		const tubegeometry2 = new THREE.CylinderGeometry(30, 30, 450, 25);
		const d13mesh = new THREE.Mesh(tubegeometry2, dmaterial);
		d13mesh.rotateX(Math.PI * 0.5);
		d13mesh.rotateY(Math.PI * 1);
		d13mesh.rotateZ(Math.PI * 0.5);
		d13mesh.position.set(-200, 400 , -930);		
		threeHCSR.add(d13mesh);			
		
		//sferacopritubo
		const sphereGeometry = new THREE.SphereGeometry(30, 30, 30);
		const d14mesh = new THREE.Mesh(sphereGeometry, dmaterial);
		d14mesh.position.set(-430, 400 , -930);		
		threeHCSR.add(d14mesh);			

		
		//up3a
		const udgeometry2h = new THREE.CylinderGeometry(30, 30, 400, 25);
		const d16mesh = new THREE.Mesh(udgeometry2h, dmaterial);
		d16mesh.rotateX(Math.PI * 0.5);
		d16mesh.rotateY(Math.PI * 1);
		d16mesh.rotateZ(Math.PI * 1);
		d16mesh.position.set(0, 650 , -900);		
		threeHCSR.add(d16mesh);		
				
		
		//up3c
		const udgeometry2m = new THREE.CylinderGeometry(30, 30, 400, 25);
		const d18mesh = new THREE.Mesh(udgeometry2m, dmaterial);
		d18mesh.rotateX(Math.PI * 0.5);
		d18mesh.rotateY(Math.PI * 0.75);
		d18mesh.rotateZ(Math.PI * 1.5);
		d18mesh.position.set(-140, 790 , -1100);		
		threeHCSR.add(d18mesh);			
 
		//sferacopritubo laterale
		const sphereGeometry2 = new THREE.SphereGeometry(30, 30, 30);
		const d19mesh = new THREE.Mesh(sphereGeometry2, dmaterial);
		d19mesh.position.set(0, 650 , -800);		
		threeHCSR.add(d19mesh);			

		
		//cubo1
		const cube1 = new THREE.BoxGeometry(440, 240, 240);
		const d22mesh = new THREE.Mesh(cube1, dmaterial);
		d22mesh.position.set(-800, -400 , 0);
		threeHCSR.add(d22mesh);	
		
		
		//tubilaterali2a
		const lateraltube2a = new THREE.CylinderGeometry(140, 140, 200, 25);
		const d24mesh = new THREE.Mesh(lateraltube2a, dmaterial);
		d24mesh.rotateX(Math.PI * 0.5);
		d24mesh.rotateY(Math.PI * 1);
		d24mesh.rotateZ(Math.PI * 0.5);
		d24mesh.position.set(-650, 0 , 0);		
		threeHCSR.add(d24mesh);			

		
		//tubilaterali3a
		const lateraltube3a = new THREE.CylinderGeometry(180, 180, 110, 25);
		const d26mesh = new THREE.Mesh(lateraltube3a, dmaterial);
		d26mesh.rotateX(Math.PI * 0.5);
		d26mesh.rotateY(Math.PI * 1);
		d26mesh.rotateZ(Math.PI * 0.5);
		d26mesh.position.set(-680, 520 , -200);	
		threeHCSR.add(d26mesh);	
		threeHCSR.scale.set(0.7, 0.7, 0.7);
		
		return threeHCSR;
	}
	// scw
	export function scw(param) {
		const length = param && typeof param.length == "number"? param.length: 2200;
		const SCWSR = new THREE.Object3D();
		const dmaterial = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
		const dmaterial2 = new THREE.MeshLambertMaterial({color: 0xffffff});
		const d1geometry = new THREE.CylinderGeometry(700, 700, length, 35);
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		SCWSR.add(d1mesh);				
		SCWSR.rotateX(Math.PI * 0.5);
		SCWSR.rotatedX = true;	

		// left
		const lrgeometry = new THREE.CylinderGeometry(750, 750, 100, 65);
		const d2mesh = new THREE.Mesh(lrgeometry, dmaterial);
		d2mesh.position.set(0, -length/2-50, 0);
		SCWSR.add(d2mesh);
	

		// right
		const d3mesh = new THREE.Mesh(lrgeometry, dmaterial);
		d3mesh.position.set(0, length/2+50, 0);
		SCWSR.add(d3mesh);
		
		// up1
		const udgeometry = new THREE.CylinderGeometry(200, 200, 700, 25);
		const d4mesh = new THREE.Mesh(udgeometry, dmaterial);
		d4mesh.rotateX(Math.PI * 0.5);
		d4mesh.rotateY(Math.PI * 1);
		d4mesh.rotateZ(Math.PI * 1);
		d4mesh.position.set(0, -600 , -770);		
		SCWSR.add(d4mesh);

		// up2
		const udgeometry2 = new THREE.CylinderGeometry(200, 200, 400, 25);
		const d5mesh = new THREE.Mesh(udgeometry2, dmaterial);
		d5mesh.rotateX(Math.PI * 0.5);
		d5mesh.rotateY(Math.PI * 1);
		d5mesh.rotateZ(Math.PI * 1);
		d5mesh.position.set(0, 0 , -730);		
		SCWSR.add(d5mesh);

		//up3
		const udgeometry2g = new THREE.CylinderGeometry(200, 200, 700, 25);
		const d6mesh = new THREE.Mesh(udgeometry2g, dmaterial);
		d6mesh.rotateX(Math.PI * 0.5);
		d6mesh.rotateY(Math.PI * 1);
		d6mesh.rotateZ(Math.PI * 1);
		d6mesh.position.set(0, 600 , -730);		
		SCWSR.add(d6mesh);	
		
		//cubo1
		const cube1 = new THREE.BoxGeometry(150, 150, 150);
		const d7mesh = new THREE.Mesh(cube1, dmaterial2);
		d7mesh.position.set(-610, 0 , -200);
		SCWSR.add(d7mesh);	
		
		SCWSR.scale.set(0.7, 0.7, 0.7);
		return SCWSR;
	}

	// Current Monitor	
	export function cm(param) {
		const cmContiner = new THREE.Object3D();
		const cmObject = new THREE.Object3D();
		
		//cylinder mid
		const cylgeometry = new THREE.CylinderGeometry(115, 115, 30, 20);
		const cylmaterial = new THREE.MeshLambertMaterial({color: 0x00d0ff});
		const cylmesh = new THREE.Mesh(cylgeometry, cylmaterial);
		cylmesh.rotateX(Math.PI * 1);
		cylmesh.rotateY(Math.PI * 1.0);
		cylmesh.rotateZ(Math.PI * 1.5);
		cylmesh.position.set(0, 0 , 0);			
		cmObject.add(cylmesh);			
		
		//cylinder center
		const cyl2geometry = new THREE.CylinderGeometry(75, 75, 31, 20);
		const cyl2material = new THREE.MeshLambertMaterial({color: 0x2d2b70});
		const cyl2mesh = new THREE.Mesh(cyl2geometry, cyl2material);
		cyl2mesh.rotateX(Math.PI * 1);
		cyl2mesh.rotateY(Math.PI * 1.0);
		cyl2mesh.rotateZ(Math.PI * 1.5);
		cyl2mesh.position.set(0, 0 , 0);			
		cmObject.add(cyl2mesh);			
		
			
		cmObject.scale.setX(1);
		cmObject.scale.setY(1);
		cmObject.scale.setZ(1);
		cmObject.position.set(0, 0, 0);
		cmContiner.add(cmObject);
		cmContiner.rotateY(Math.PI * -0.5);		
		
		
		return cmContiner;
	}
	// corrector
	export function correctorfast(param) {
		const width=250, height=250, depth=200;
		const correctorObject = new THREE.Object3D();
		const correctormaterial = new THREE.MeshLambertMaterial({color: 0x4040b0});
		const correctorgeometry = new THREE.BoxGeometry(width, height, depth);
		const correctormesh = new THREE.Mesh(correctorgeometry, correctormaterial);
		correctorObject.add(correctormesh);
		return correctorObject;
	}

	// Bending Dipole
	export function dipolefast(length=1400) {
		const dipoleMaster = new THREE.Object3D();
		const dmaterial = new THREE.MeshLambertMaterial({color: 0x00c000});
		const d1geometry = new THREE.BoxGeometry(500, 700, length);
		const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
		d1mesh.position.set(100, 100, 0);
		dipoleMaster.add(d1mesh);				
		const dipoleCoil = new THREE.Object3D();
		const coilmaterial = new THREE.MeshLambertMaterial({color: 0xc08000});
		const d4geometry = new THREE.BoxGeometry(100, 100, length);
		const d4mesh = new THREE.Mesh(d4geometry, coilmaterial);
		d4mesh.position.set(-200, 0, 0);
		dipoleCoil.add(d4mesh);
		const d2geometry = new THREE.CylinderGeometry(150 /*radiusTop*/, 150 /*radiusBottom*/, 100 /*height*/, 20 /*radialSegments*/, 2 /*heightSegments*/, false /*openEnded*/,Math.PI * 1.5 /*thetaStart*/, Math.PI * 1 /*thetaLength*/);
		const d2mesh = new THREE.Mesh(d2geometry, coilmaterial);
		d2mesh.position.set(-100, 0, length/2);
		dipoleCoil.add(d2mesh);
		const d3geometry = new THREE.CylinderGeometry(150 /*radiusTop*/, 150 /*radiusBottom*/, 100 /*height*/, 20 /*radialSegments*/, 2 /*heightSegments*/, false /*openEnded*/,Math.PI * 0.5 /*thetaStart*/, Math.PI * 1 /*thetaLength*/);
		const d3mesh = new THREE.Mesh(d3geometry, coilmaterial);
		d3mesh.position.set(-100, 0, -length/2);
		dipoleCoil.add(d3mesh);
		dipoleCoil.position.set(0, 0, 0);
		dipoleMaster.add(dipoleCoil);
		const dipoleCoil2 = dipoleCoil.clone();
		dipoleCoil2.position.set(0, 200, 0);
		dipoleMaster.add(dipoleCoil2);
		return dipoleMaster;
	}

	// Quadrupole
	export function quadrupole(param) {
		const length=260;
		const quadrupoleMaster = new THREE.Object3D();
		const chamberMaterial = new THREE.MeshLambertMaterial({color: 0xc0c0c0});
		const quadrupolematerial = new THREE.MeshLambertMaterial({color: 0xff7050});
		const quadrupolegeometry = new THREE.CylinderGeometry(250, 250, length, 8);
		const quadrupolemesh = new THREE.Mesh(quadrupolegeometry, quadrupolematerial);
		quadrupoleMaster.add(quadrupolemesh);
		const quadrupole2geometry = new THREE.CylinderGeometry(290, 290, 30, 50);
		const quadrupole2mesh = new THREE.Mesh(quadrupole2geometry, chamberMaterial);
		quadrupole2mesh.position.set(0, -length/2-15, 0);
		quadrupoleMaster.add(quadrupole2mesh);
		const quadrupole3mesh = new THREE.Mesh(quadrupole2geometry, chamberMaterial);
		quadrupole3mesh.position.set(0, length/2+15, 0);
		quadrupoleMaster.add(quadrupole3mesh);
		quadrupoleMaster.rotateX(Math.PI * 0.5);
		quadrupoleMaster.rotatedX = true;
		const qedges = new THREE.EdgesGeometry(quadrupolegeometry);
		const lineMaterial = new THREE.LineBasicMaterial({color: 0x600000});
		lineMaterial.linewidth = 4;
		const qline = new THREE.LineSegments(qedges, lineMaterial);
		quadrupoleMaster.add(qline);
		return quadrupoleMaster;
	}


	// Sextupole
	export function sextupole(param) {
		const length=230;
		const sextupoleMaster = new THREE.Object3D();
		const chamberMaterial = new THREE.MeshLambertMaterial({color: 0xc0c0c0});
		const sextupolematerial = new THREE.MeshLambertMaterial({color: 0xffff00});
		const sextupolegeometry = new THREE.CylinderGeometry(250, 250, length, 6);
		const sextupolemesh = new THREE.Mesh(sextupolegeometry, sextupolematerial);
		sextupoleMaster.add(sextupolemesh);
		const sextupole2geometry = new THREE.CylinderGeometry(290, 290, 30, 50);
		const sextupole2mesh = new THREE.Mesh(sextupole2geometry, chamberMaterial);
		sextupole2mesh.position.set(0, -length/2-15, 0);
		sextupoleMaster.add(sextupole2mesh);
		const sextupole3mesh = new THREE.Mesh(sextupole2geometry, chamberMaterial);
		sextupole3mesh.position.set(0, length/2+15, 0);
		sextupoleMaster.add(sextupole3mesh);
		sextupoleMaster.rotateX(Math.PI * 0.5);
		sextupoleMaster.rotatedX = true;
		const sedges = new THREE.EdgesGeometry(sextupolegeometry);
		const sline = new THREE.LineSegments(sedges, new THREE.LineBasicMaterial({color: 0x808000}));
		sextupoleMaster.add(sline);
		return sextupoleMaster;
	}


	// undulator
	export function undulator(param) {
		const length=1300;
		const undulatorMaster = new THREE.Object3D();
		const chamberMaterial = new THREE.MeshLambertMaterial({color: 0xc0c0c0});
		const loader = new THREE.TextureLoader();
		const texture = loader.load('./components/undulator.png');
		const textureTop = texture.clone();
		const textureSide = texture.clone();
		textureTop.wrapS = textureTop.wrapT = THREE.RepeatWrapping;
		textureTop.repeat.set(30, 1);
		textureSide.offset.set(0.95, 0.95);
		const face = new THREE.MeshBasicMaterial({map: texture});
		const faceTop = new THREE.MeshBasicMaterial({map: textureTop});
		const faceSide = new THREE.MeshBasicMaterial({map: textureSide});
		const materials = [faceTop,faceTop,chamberMaterial,chamberMaterial,chamberMaterial,chamberMaterial];
		const ugeometry = new THREE.BoxGeometry(400, 400, length);
		const umesh = new THREE.Mesh(ugeometry, materials);
		undulatorMaster.add(umesh);				
		const qedges = new THREE.EdgesGeometry(ugeometry);
		const lineMaterial = new THREE.LineBasicMaterial({color: 0x000000});
		lineMaterial.linewidth = 4;
		const qline = new THREE.LineSegments(qedges, lineMaterial);
		undulatorMaster.add(qline);
		return undulatorMaster;
	}

	// wiggler
	export function wiggler(param) {
		const length=1300;
		const wigglerMaster = new THREE.Object3D();
		const chamberMaterial = new THREE.MeshLambertMaterial({color: 0xf3a1eb});
		const loader = new THREE.TextureLoader();
		const texture = loader.load('./components/wiggler.png');
		const textureTop = texture.clone();
		const textureSide = texture.clone();
		textureTop.wrapS = textureTop.wrapT = THREE.RepeatWrapping;
		textureTop.repeat.set(30, 1);
		textureSide.offset.set(0.95, 0.95);
		const face = new THREE.MeshBasicMaterial({map: texture});
		const faceTop = new THREE.MeshBasicMaterial({map: textureTop});
		const faceSide = new THREE.MeshBasicMaterial({map: textureSide});
		const materials = [faceTop,faceTop,chamberMaterial,chamberMaterial,chamberMaterial,chamberMaterial];
		const ugeometry = new THREE.BoxGeometry(400, 400, length);
		const umesh = new THREE.Mesh(ugeometry, materials);
		wigglerMaster.add(umesh);				
		const qedges = new THREE.EdgesGeometry(ugeometry);
		const lineMaterial = new THREE.LineBasicMaterial({color: 0x000000});
		lineMaterial.linewidth = 4;
		const qline = new THREE.LineSegments(qedges, lineMaterial);
		wigglerMaster.add(qline);
		return wigglerMaster;
	}

	// Beam Loss Monitor
	export function blm(param) {
		const length=100, dcolor=0xffffff, width=100;
		const cMaster = new THREE.Object3D();
		const d1geometry = new THREE.BoxGeometry(length, width, width);
		const material = new THREE.MeshLambertMaterial({color: dcolor});
		const d1mesh = new THREE.Mesh(d1geometry, material);
		d1mesh.position.set(-length/2, 0, 0);
		cMaster.add(d1mesh);
		return cMaster;
	}
	function blmColor(val) {
		if (document.location.search.indexOf('demo')>-1) {
			if (val < 0) return 'white';
			else if (val < 1) return 'yellow';
			else if (val < 2) return 'orange';
			else if (val < 3) return 'red';
			else return 'violet';
		}
		if (val < 20) return 'white';
		else if (val < 50) return 'yellow';
		else if (val < 100) return 'orange';
		else if (val < 200) return 'red';
		else return 'violet';
	}
	export function blmUpdate(c, val, direction, width=100) {
		if (typeof val !== 'number') val = 0;
		const blm = c.children[0];
		blm.material.dispose();
		blm.material = new THREE.MeshLambertMaterial({color: blmColor(val)});
		blm.geometry.dispose();
		const histogramFactor = document.location.search.indexOf('demo')>-1? 1000: 50;
		blm.geometry = new THREE.BoxGeometry(width+val*histogramFactor, width, width);
		// console.log('blmUpdate()', c, val, direction, width, width+val*histogramFactor);
		blm.position.set(direction*(width+val*histogramFactor)/2, 0, 0);
		blm.geometry.computeVertexNormals();
		blm.geometry.normalsNeedUpdate = true;
		blm.geometry.verticesNeedUpdate = true;
		blm.geometry.dynamic = true;
	}

	// BeamlinesUP
	export function beamlineUp(param) {
		// console.log('beamlineUp()',param);
		const continer = new THREE.Object3D();
		const text=param.name, height=50;
		let length = param && param.labelReverse && param.labelReverse[1]? param.labelReverse[1]: 200;
		const depth = param && param.labelReverse && param.labelReverse[2]? param.labelReverse[2]: 18000*(param.labelReverse? -1: 1);
		let size=500;
		const materialBase = new THREE.MeshLambertMaterial({color: 0xff0000});
		const objLabel = new THREE.Object3D();
		let group, textMesh, textGeo, materials;
		let textheight = 50;
		let pos = [];
		let rot = [];
		const hover = 30;
		const curveSegments = 8;
		let font;
		function insertText(isize, ipos, irot) {
			group = new THREE.Group();
			objLabel.add( group );
			size = isize;
			pos = ipos;
			rot = irot;
			loadFont();
		}
		function loadFont() {
			const loader = new FontLoader();
			// https://threejs.org/examples/fonts/droid_sans_bold.typeface.json
			loader.load( './fonts/droid_sans_bold.typeface.json', function ( response ) {
				font = response;
				refreshText();
			} );
		}
		function createText() {
			textGeo = new TextGeometry(text, {font: font, size: size, height: textheight, curveSegments: curveSegments});
			textGeo.computeBoundingBox();
			const centerOffset = - 0.5 * ( textGeo.boundingBox.max.x - textGeo.boundingBox.min.x );
			textMesh = new THREE.Mesh( textGeo, materialBase );
			textMesh.position.x = pos[0];
			textMesh.position.y = pos[1];
			textMesh.position.z = pos[2];
			textMesh.rotation.x = rot[0];
			textMesh.rotation.y = rot[1];
			textMesh.rotation.z = rot[2];
			group.add( textMesh );
		}
		function refreshText() {
			group.remove( textMesh );
			if ( ! text ) return;
			createText();
		}
		insertText(size, [length, height, depth], [Math.PI*0.5, Math.PI, Math.PI*0.5]);
		continer.add(objLabel);
		length=30000;
		const material = new THREE.MeshBasicMaterial({color: 0xA5D9FD, transparent: true, opacity: 0.4, side: THREE.DoubleSide});
		const geometry = new THREE.CylinderGeometry(1500*(param.labelReverse? 0: 1) /*radiusTop*/, 1500*(param.labelReverse? 1: 0) /*radiusBottom*/, length /*height*/, 2 /*radialSegments*/, 2 /*heightSegments*/, true /*openEnded*/,0 /*thetaStart*/, Math.PI * 0.5 /*thetaLength*/);
		const beamlineUpMesh = new THREE.Mesh(geometry, material);
		beamlineUpMesh.position.set(0, 0, length/2*(param.labelReverse? -1: 1));	
		beamlineUpMesh.rotateX(Math.PI * 0.5);
		beamlineUpMesh.rotateY(Math.PI * 0.5);
		beamlineUpMesh.scale.setX(0.001);
		continer.add(beamlineUpMesh);
		return continer;
	}
	// BeamlinesDown
	export function beamlineDown(param) {
		// console.log('beamlineDown()',param);
		const continer = new THREE.Object3D();
		const text=param.name, height=50;
		let length = param && param.labelReverse && param.labelReverse[1]? param.labelReverse[1]: -700;
		const depth = param && param.labelReverse && param.labelReverse[2]? param.labelReverse[2]: 18000*(param.labelReverse? -1: 1);
		let size=500;
		const materialBase = new THREE.MeshLambertMaterial({color: 0xff0000});
		const objLabel = new THREE.Object3D();
		let group, textMesh, textGeo, materials;
		let textheight = 50;
		let pos = [];
		let rot = [];
		const hover = 30;
		const curveSegments = 8;
		let font;
		function insertText(isize, ipos, irot) {
			group = new THREE.Group();
			objLabel.add( group );
			size = isize;
			pos = ipos;
			rot = irot;
			loadFont();
		}
		function loadFont() {
			const loader = new FontLoader();
			// https://threejs.org/examples/fonts/droid_sans_bold.typeface.json
			loader.load( './fonts/droid_sans_bold.typeface.json', function ( response ) {
				font = response;
				refreshText();
			} );
		}
		function createText() {
			textGeo = new TextGeometry(text, {font: font, size: size, height: textheight, curveSegments: curveSegments});
			textGeo.computeBoundingBox();
			const centerOffset = - 0.5 * ( textGeo.boundingBox.max.x - textGeo.boundingBox.min.x );
			textMesh = new THREE.Mesh( textGeo, materialBase );
			textMesh.position.x = pos[0];
			textMesh.position.y = pos[1];
			textMesh.position.z = pos[2];
			textMesh.rotation.x = rot[0];
			textMesh.rotation.y = rot[1];
			textMesh.rotation.z = rot[2];
			group.add( textMesh );
		}
		function refreshText() {
			group.remove( textMesh );
			if ( ! text ) return;
			createText();
		}
		insertText(size, [length, height, depth], [Math.PI*0.5, Math.PI, Math.PI*0.5]);
		continer.add(objLabel);
		length=30000;
		const material = new THREE.MeshBasicMaterial({color: 'limegreen', transparent: true, opacity: 0.2, side: THREE.DoubleSide});
		const geometry = new THREE.CylinderGeometry(1500*(param.labelReverse? 0: 1) /*radiusTop*/, 1500*(param.labelReverse? 1: 0) /*radiusBottom*/, length /*height*/, 2 /*radialSegments*/, 2 /*heightSegments*/, true /*openEnded*/,Math.PI /*thetaStart*/, Math.PI * 0.5 /*thetaLength*/);
		const beamlineUpMesh = new THREE.Mesh(geometry, material);
		beamlineUpMesh.position.set(0, 0, length/2*(param.labelReverse? -1: 1));	
		beamlineUpMesh.rotateX(Math.PI * 0.5);
		beamlineUpMesh.rotateY(Math.PI * 0.5);
		beamlineUpMesh.scale.setX(0.001);
		continer.add(beamlineUpMesh);
		return continer;
	}

	// scraper
	export function scraper(param) {
		const ScraperContiner = new THREE.Object3D();
		const ScraperObject = new THREE.Object3D();
		
		// box esterno
		const boxmaterial = new THREE.MeshLambertMaterial({color: 0xd6d6d6});
		const boxgeometry = new THREE.BoxGeometry(140, 40, 400);
		const d1mesh = new THREE.Mesh(boxgeometry, boxmaterial);
		d1mesh.position.set(0, 0, 0);
		d1mesh.rotateX(Math.PI * 1.250);
		d1mesh.rotateY(Math.PI * 1.5);
		d1mesh.rotateZ(Math.PI * 1.25);	
		d1mesh.position.set(0, 0, 0);
		ScraperObject.add(d1mesh);
		
		// box lato1
		const sidematerial2 = new THREE.MeshLambertMaterial({color: 0xd6d6d6});
		const sidegeometry2 = new THREE.BoxGeometry(140, 20, 140 );
		const d2mesh = new THREE.Mesh(sidegeometry2, sidematerial2);
		d2mesh.position.set(0, 0, 0);
		d2mesh.rotateX(Math.PI * 1.5);
		d2mesh.rotateY(Math.PI * 1);
		d2mesh.rotateZ(Math.PI * 1.5);	
		d2mesh.position.set(-191, 50, 0);
		ScraperObject.add(d2mesh);		
		
		// box lato2
		const sidematerial3 = new THREE.MeshLambertMaterial({color: 0xd6d6d6});
		const sidegeometry3 = new THREE.BoxGeometry(140, 20, 140 );
		const d3mesh = new THREE.Mesh(sidegeometry3, sidematerial3);
		d3mesh.position.set(0, 0, 0);
		d3mesh.rotateX(Math.PI * 1.5);
		d3mesh.rotateY(Math.PI * 1);
		d3mesh.rotateZ(Math.PI * 1.5);	
		d3mesh.position.set(191, 50, 0);
		ScraperObject.add(d3mesh);		
		
		// box esterno2
		const boxgeometry2 = new THREE.BoxGeometry(140, 40, 400);
		const d5mesh = new THREE.Mesh(boxgeometry2, boxmaterial);
		d5mesh.position.set(0, 0, 0);
		d5mesh.rotateX(Math.PI * 1.250);
		d5mesh.rotateY(Math.PI * 1.5);
		d5mesh.rotateZ(Math.PI * 1.25);	
		d5mesh.position.set(0, 100, 0);
		ScraperObject.add(d5mesh);		
		
		//cilinder
		const cilgeometry = new THREE.CylinderGeometry(50, 50, 340, 25);
		const cilmaterial = new THREE.MeshLambertMaterial({color: 0xc4c4c4});
		const d4mesh = new THREE.Mesh(cilgeometry, cilmaterial);
		d4mesh.rotateX(Math.PI * 1);
		d4mesh.rotateY(Math.PI * 1.5);
		d4mesh.rotateZ(Math.PI * 1.5);
		d4mesh.position.set(0,50 , 0);			
		ScraperObject.add(d4mesh);	
		
		
		
		ScraperObject.position.set(0, -150, 0);
		ScraperObject.scale.setX(3);
		ScraperObject.scale.setY(3);
		ScraperObject.scale.setZ(1);
		ScraperContiner.add(ScraperObject);
		ScraperContiner.rotateY(Math.PI * -1);		
		
		
		return ScraperContiner;
	}
	// lmbf
		export function lmbf(param) {
			const length=300;
			const lmbfSR = new THREE.Object3D();
			const lmbfContainer = new THREE.Object3D();
			
			const dmaterial = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
			const dmaterial2 = new THREE.MeshLambertMaterial({color: 0x969696});
			
			const d1geometry = new THREE.CylinderGeometry(100, 100, 250, 25);
			const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
			d1mesh.position.set(0, 0, 0);
			lmbfContainer.add(d1mesh);				
			lmbfContainer.rotateX(Math.PI * 0.5);
			lmbfContainer.rotatedX = true;	

			// left
			const d2geometry = new THREE.CylinderGeometry(100, 100, 10, 25);
			const d2mesh = new THREE.Mesh(d2geometry, dmaterial2);
			d2mesh.position.set(0, -130, 0);
			lmbfContainer.add(d2mesh);

			// right
			const d3geometry = new THREE.CylinderGeometry(100, 100, 10, 25);
			const d3mesh = new THREE.Mesh(d3geometry, dmaterial2);
			d3mesh.position.set(0, 130, 0);
			lmbfContainer.add(d3mesh);

			// up1
			const d4geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d4mesh = new THREE.Mesh(d4geometry, dmaterial2);
			d4mesh.rotateX(Math.PI * 0.5);
			d4mesh.rotateY(Math.PI * 1);
			d4mesh.rotateZ(Math.PI * 1);
			d4mesh.position.set(0, -80 , 0);		
			lmbfContainer.add(d4mesh);

			// up2
			const d5geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d5mesh = new THREE.Mesh(d5geometry, dmaterial2);
			d5mesh.rotateX(Math.PI * 0.5);
			d5mesh.rotateY(Math.PI * 1);
			d5mesh.rotateZ(Math.PI * 1);
			d5mesh.position.set(0, 80 , 0);		
			lmbfContainer.add(d5mesh);
			
			// lef1
			const d6geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d6mesh = new THREE.Mesh(d6geometry, dmaterial2);
			d6mesh.rotateX(Math.PI * 0.5);
			d6mesh.rotateY(Math.PI * 1);
			d6mesh.rotateZ(Math.PI * 0.5);
			d6mesh.position.set(0, -80 , 0);		
			lmbfContainer.add(d6mesh);

			// right2
			const d7geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d7mesh = new THREE.Mesh(d7geometry, dmaterial2);
			d7mesh.rotateX(Math.PI * 0.5);
			d7mesh.rotateY(Math.PI * 1);
			d7mesh.rotateZ(Math.PI * 0.5);
			d7mesh.position.set(0, 80 , 0);		
			lmbfContainer.add(d7mesh);			
			
			lmbfContainer.position.set(0, 0, 0);
		    lmbfSR.add(lmbfContainer);
		    lmbfSR.rotateY(Math.PI * 1);
			return lmbfSR;
		}
	// tmbf
		export function tmbf(param) {
			const length=600;
			const tmbfSR = new THREE.Object3D();
			const tmbfContainer = new THREE.Object3D();
			
			const dmaterial = new THREE.MeshLambertMaterial({color: 0xffd700});
			const dmaterial2 = new THREE.MeshLambertMaterial({color: 0x969696});
			const dmaterialsilver = new THREE.MeshLambertMaterial({color: 0xf0f0f0});
			
			const d1geometry = new THREE.CylinderGeometry(100, 100, 600, 25);
			const d1mesh = new THREE.Mesh(d1geometry, dmaterial);
			d1mesh.position.set(0, 0, 0);
			tmbfContainer.add(d1mesh);				
			tmbfContainer.rotateX(Math.PI * 0.5);
			tmbfContainer.rotatedX = true;	

			// coverleft
			const d2geometry = new THREE.CylinderGeometry(130, 130, 20, 25);
			const d2mesh = new THREE.Mesh(d2geometry, dmaterial);
			d2mesh.position.set(0, -300, 0);
			tmbfContainer.add(d2mesh);
			

			//covercenter
			const dmgeometry = new THREE.CylinderGeometry(130, 130, 40, 25);
			const dmmesh = new THREE.Mesh(dmgeometry, dmaterial);
			dmmesh.position.set(0, 0, 0);
			tmbfContainer.add(dmmesh);			
			
			//coverright
			const d3geometry = new THREE.CylinderGeometry(130, 130, 20, 25);
			const d3mesh = new THREE.Mesh(d3geometry, dmaterial);
			d3mesh.position.set(0, 300, 0);
			tmbfContainer.add(d3mesh);
			
			// up1
			const d4geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d4mesh = new THREE.Mesh(d4geometry, dmaterial2);
			d4mesh.rotateX(Math.PI * 0.5);
			d4mesh.rotateY(Math.PI * 1);
			d4mesh.rotateZ(Math.PI * 1);
			d4mesh.position.set(0, -80 , 0);		
			tmbfContainer.add(d4mesh);

			// up2
			const d5geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d5mesh = new THREE.Mesh(d5geometry, dmaterial2);
			d5mesh.rotateX(Math.PI * 0.5);
			d5mesh.rotateY(Math.PI * 1);
			d5mesh.rotateZ(Math.PI * 1);
			d5mesh.position.set(0, -200 , 0);		
			tmbfContainer.add(d5mesh);
			
			// lef1
			const d6geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d6mesh = new THREE.Mesh(d6geometry, dmaterial2);
			d6mesh.rotateX(Math.PI * 0.5);
			d6mesh.rotateY(Math.PI * 1);
			d6mesh.rotateZ(Math.PI * 0.5);
			d6mesh.position.set(0, 80 , 0);		
			tmbfContainer.add(d6mesh);

			// right2
			const d7geometry = new THREE.CylinderGeometry(10, 10, 250, 25);
			const d7mesh = new THREE.Mesh(d7geometry, dmaterial2);
			d7mesh.rotateX(Math.PI * 0.5);
			d7mesh.rotateY(Math.PI * 1);
			d7mesh.rotateZ(Math.PI * 0.5);
			d7mesh.position.set(0, 200 , 0);		
			tmbfContainer.add(d7mesh);			
			
			tmbfContainer.position.set(0, 0, 0);
		    tmbfSR.add(tmbfContainer);
		    tmbfSR.rotateY(Math.PI * 1);
			return tmbfSR;
		}
	// bst
	export function bst(param) {
		const bstObject = new THREE.Object3D();
		const bstContiner = new THREE.Object3D();
		const materialGrey = new THREE.MeshLambertMaterial({color: 0xc0c0c0});

		const d1geometry = new THREE.CylinderGeometry(25, 25, 200, 25);
		const d1mesh = new THREE.Mesh(d1geometry, materialGrey);
		d1mesh.position.set(0, 190, 0);
		bstObject.add(d1mesh);				
		bstObject.rotateX(Math.PI * 1);

		// circ base
		const d2geometry = new THREE.CylinderGeometry(50, 50, 80, 15);
		const d2mesh = new THREE.Mesh(d2geometry, materialGrey);
		d2mesh.position.set(0, 290, 0);
		d2mesh.rotateX(Math.PI * 1);
		d2mesh.rotateY(Math.PI * 1);
		d2mesh.rotateZ(Math.PI * 1.5);		
		bstObject.add(d2mesh);

		// z
		const d4geometry = new THREE.CylinderGeometry(40, 40, 120, 15);
		const d4mesh = new THREE.Mesh(d4geometry, materialGrey);
		d4mesh.rotateX(Math.PI * 1);
		d4mesh.rotateY(Math.PI * 1);
		d4mesh.rotateZ(Math.PI * 1);
		d4mesh.position.set(0, 120 , 0);
		d4mesh.colorable = true;
		bstObject.add(d4mesh);

		// cil Med
		const d6geometry = new THREE.CylinderGeometry(70, 70, 120, 15);
		const d6mesh = new THREE.Mesh(d6geometry, materialGrey);
		d6mesh.rotateX(Math.PI * 1);
		d6mesh.rotateY(Math.PI * 1);
		d6mesh.rotateZ(Math.PI * 1);
		d6mesh.position.set(0, 0 , 0);
		d6mesh.colorable = true;
		bstObject.add(d6mesh);

		// cil low
		const d7geometry = new THREE.CylinderGeometry(50, 50, 120, 15);
		const d7mesh = new THREE.Mesh(d7geometry, materialGrey);
		d7mesh.rotateX(Math.PI * 1);
		d7mesh.rotateY(Math.PI * 1);
		d7mesh.rotateZ(Math.PI * 1);
		d7mesh.position.set(0, -100 , 0);
		d7mesh.colorable = true;
		bstObject.add(d7mesh);

		bstObject.position.set(0, 280, 0);
		bstContiner.add(bstObject);
		bstContiner.rotateY(Math.PI * -0.5);
		return bstContiner;
	}
	export function bstUpdate(c, color) {
		const vlv = c.children[0].children;
		for (let i=0; i<vlv.length; i++) {
			if (vlv[i].colorable) {
				vlv[i].material.dispose();
				vlv[i].material = new THREE.MeshLambertMaterial({color: color});
			}
		}
	}
	// valve
	export function vlv(param) {
		const vlvContiner = new THREE.Object3D();
		const vlvObject = new THREE.Object3D();
		const blackboxmaterial = new THREE.MeshLambertMaterial({color: 0x636363});
		//cylinder
		const cylgeometry = new THREE.CylinderGeometry(25, 25, 100, 15);
		const cylmaterial = new THREE.MeshLambertMaterial({color: 0xc7c7c7});
		const cylmesh = new THREE.Mesh(cylgeometry, cylmaterial);
		cylmesh.rotateX(Math.PI * 1);
		cylmesh.rotateY(Math.PI * 1.0);
		cylmesh.rotateZ(Math.PI * 1.5);
		cylmesh.position.set(0, 0 , 0);			
		vlvObject.add(cylmesh);			
		
		// base1
		const vlvgeometry = new THREE.BoxGeometry(30, 160, 60 );
		const vlvmaterial = new THREE.MeshLambertMaterial({color: 0xc7c7c7});		
		const vlv1mesh = new THREE.Mesh(vlvgeometry, vlvmaterial);
		vlv1mesh.position.set(0, 50, 0);
		vlv1mesh.colorable = true;
		vlvObject.add(vlv1mesh);		
		
		// hat1
		const hatmaterial = new THREE.MeshLambertMaterial({color: 0xc7c7c7});
		const hatgeometry = new THREE.BoxGeometry(50, 20, 100 );
		const hatmesh = new THREE.Mesh(hatgeometry, hatmaterial);
		hatmesh.colorable = true;
		hatmesh.position.set(0, 120, 0);
		vlvObject.add(hatmesh);	
		
		
		//TRIANGLE1
		const trianglegeometry = new THREE.CylinderGeometry(18, 18, 32, 3);
		const trianglegeometrymesh = new THREE.Mesh(trianglegeometry, blackboxmaterial);
		trianglegeometrymesh.rotateX(Math.PI * 0);
		trianglegeometrymesh.rotateY(Math.PI * 0);
		trianglegeometrymesh.rotateZ(Math.PI * 0.5);		
		trianglegeometrymesh.position.set(0, 60 , -17);		
		vlvObject.add(trianglegeometrymesh);	
		
		//TRIANGLE2
		const trianglegeometry2 = new THREE.CylinderGeometry(18, 18, 32, 3);
		const triangle2geometrymesh = new THREE.Mesh(trianglegeometry2, blackboxmaterial);
		triangle2geometrymesh.rotateX(Math.PI * 0);
		triangle2geometrymesh.rotateY(Math.PI * 1);
		triangle2geometrymesh.rotateZ(Math.PI * 0.5);		
		triangle2geometrymesh.position.set(0, 60 ,17);		
		vlvObject.add(triangle2geometrymesh);		
		
		vlvObject.position.set(0, 0, 0);
		vlvObject.rotateY(Math.PI * -0.5);
		vlvContiner.add(vlvObject);

		return vlvContiner;
	}
	export function vlvUpdate(c, color) {
		const vlv = c.children[0].children;
		for (let i=0; i<vlv.length; i++) {
			if (vlv[i].colorable) {
				vlv[i].material.dispose();
				vlv[i].material = new THREE.MeshLambertMaterial({color: color});
			}
		}
	}
